-- $LastChangedDate: 2008-11-09 08:39:03 -0500 (Sun, 09 Nov 2008) $
-- This chunk is called every timer-tick.
-- See module.txt for documentation and pitfalls.

-- NOP if controlled by trackball.
if ( sim.argv2["-t"] ) then
	return
end

-- Call registered timer-tick callbacks.
Timer:Tick()

--------------------------------------------------------------------------------
-- The main timer-tick handler.

-- Keyboard handler.
local keyEvent = sim.Keyboard:DequeueKey()
if ( keyEvent ) then
	Keyboard:Handler( keyEvent )
end

-- Joystick handler (button).
-- sim.Joystick:DequeueButton() returns nil or a table:
-- { buttonNum=int, buttonVal=bool }
local joyButtonEvent = sim.Joystick:DequeueButton()
if ( joyButtonEvent ) then
	local pressed = joyButtonEvent.buttonVal
	-- Apply brakes while button is pressed.
	if ( joyButtonEvent.buttonNum == defs.JOYSTICK_BUTTON_BRAKES ) then
		PlayerActor.aircraft:EnableBrakes( joyButtonEvent.buttonVal )
	elseif ( joyButtonEvent.buttonNum == defs.JOYSTICK_BUTTON_FIRE_GUN and pressed ) then
		PlayerActor:FireGun()
	elseif ( joyButtonEvent.buttonNum == defs.JOYSTICK_BUTTON_FIRE_MISSILE and pressed ) then
		PlayerActor:FireMissile()
	elseif ( joyButtonEvent.buttonNum == defs.JOYSTICK_BUTTON_WING_SWEEP_FORWARD and pressed ) then
		Joystick.repeatIncWingSweep = 1
	elseif ( joyButtonEvent.buttonNum == defs.JOYSTICK_BUTTON_WING_SWEEP_BACKWARD and pressed ) then
		Joystick.repeatIncWingSweep = -1
	end

	if ( not pressed ) then
		Joystick.repeatIncWingSweep = nil
	end
end

-- Repeat some joystick buttons until release.
if ( Joystick.repeatIncWingSweep ) then
	PlayerActor:IncWingSweep( Joystick.repeatIncWingSweep, "joy" )
end

-- Update engine sound.
local engineVolume = PlayerActor.aircraft:GetThrottle()  -- throttle equals volume {0.0,..,1.0}
if ( engineVolume < 0.05 or PlayerActor.aircraft:IfCrash() ) then
	PlayerActor.aircraft:StopEngineSound()
else
	PlayerActor.aircraft:PlayEngineSound( engineVolume )
end

-- Update stall warning sound.
if ( PlayerActor.aircraft:GetPhysics():IfStall() ) then
	sim.Sound:Play( "stall.wav.gz", sim.Sound.PLAY_LOOP, 0.6 )
else
	sim.Sound:Stop( "stall.wav.gz" )
end

-- Process commands from the simulator's "command queue".
CommandQueue:Process()  -- cmd.lua

-- CRITICAL: Collect garbage to induce freeing memory of 3D models
-- (which consume large amounts of memory).
if ( (not garbageCollectionTime) or (sim.Program:GetElapsedTime() - garbageCollectionTime > 500) ) then
	garbageCollectionTime = sim.Program:GetElapsedTime()  -- milliseconds
	collectgarbage()
end
