-- $LastChangedDate: 2008-11-12 18:30:45 -0500 (Wed, 12 Nov 2008) $
-- Weapons.

-----------
-- Radar --
-----------

Radar = {
	RANGE = Meters2Sim( defs.PLAYER_RADAR_RANGE_METERS ),
}

-- ctor
function Radar:new( actor )
	if ( self == Radar ) then
		self = lib.NewInstance( Radar )  -- not chained, need new instance
	end
	self.actor        = actor
	self.range        = Radar.RANGE
	self.enemyTeamNum = Game:OppositeTeamNum( self.actor )
	self.lockedTarget = nil
	return self
end

-- dtor (explicit)
function Radar:destroy()
	self.destroyed = true
	self.lockedTarget = nil  -- unreference Actor/Object
	self.actor = nil
end

-- Get target that radar has lock or nil.
function Radar:GetTarget()
	return self.lockedTarget
end

-- Tick method must be called by the encapsulating actor (see actor.lua).
function Radar:Tick()
-- This needs to be re-thought (it just picks the first target in range,
-- but should stay locked-on)
--[[
	if ( self.destroyed ) then
		return
	end

	-- Update list of actors being tracked by radar.
	self.lockedTarget = nil
	local pos = self.actor:GetObject():GetPosition()
	for idx,actor in pairs(Game:GetActors()) do
		-- Enemy?
		if ( actor:GetTeamNum() == self.enemyTeamNum ) then
			-- This is an enemy.  How far away is it?
			local enemyPos = actor:GetObject():GetPosition()
			local distance = Distance3( pos, enemyPos )
			if ( distance < self.range ) then
				-- Remember.
				self.lockedTarget = actor
				break
			end
		end
	end
]]--
end

-----------------
-- PlayerRadar --
-----------------

PlayerRadar = lib.DerivedClass( Radar )

-- ctor
function PlayerRadar:new( actor )
	self = lib.NewInstance( PlayerRadar )

	Radar.new( self, actor )  -- chain

	-- For debugging radar/missiles.
	if ( sim.argv2["-inf-radar"] ) then
		self.range = Meters2Sim( 10000000.0 )
	end

	return self
end

-- dtor (explicit)
function PlayerRadar:destroy()
	Radar.destroy( self )  -- chain
end

-- Tick method must be called by the encapsulating actor (see actor.lua).
function PlayerRadar:Tick()
	if ( self.destroyed ) then
		return
	end

	-- Assume no-lock.
	self.lockedTarget = nil
	sim.GameHud:RemoveRadarLock()

	-- Has player selected a target?
	local target = PlayerActor:GetSelectedTarget()
	if ( target ) then
		-- Compute distance between player and target.
		local playerPos = PlayerActor:GetObject():GetPosition()
		local targetPos = target:GetObject():GetPosition()
		local distance = Distance3( playerPos, targetPos )
		if ( distance < self.range ) then
			-- Draw HUD box with X to indicate radar-lock.
			if ( target ~= self.lockedTarget ) then  -- transition?
				PlayerActor:SelectTarget( target, true )
			end
			-- Remember.
			self.lockedTarget = target
			Hud:RemoveStatusText( defs.HUD_STATUS_NO_RADAR_LOCK )
		end
	end
end
