/*****************************************************************************
 * $LastChangedDate: 2008-11-11 09:17:33 -0500 (Tue, 11 Nov 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	ParticleSystemFX is a derivative to be contained in the FX container.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef FX_FX_PARTSYS_HH  // FX_FX twice
#define FX_FX_PARTSYS_HH 1

#include "fx/partsys.hh"

namespace fx {

////////////////////////////////////////////////////////////////////////////////
/// @brief ParticleSystemFX is a derivative to be contained in the FX container.
///
/// The base ParticleSystem is generalized, uncontained, has no expiry.
/// ParticleSystemFX is a intermediate class for developing specialized
/// particle systems for fire, explosions, etc to be contained by the FX container.
///
class ParticleSystemFX : public ParticleSystem
{
PREVENT_COPYING(ParticleSystemFX)
typedef ParticleSystem Parent;
friend class shptr<ParticleSystemFX>;
protected:  // instantiation is limited
					ParticleSystemFX( const Milliseconds lifetime,
                                      const StateBits particleStateBits = ParticleSystem::DEFAULT_STATE_BITS );
	virtual			~ParticleSystemFX();

protected:
	virtual void	Tick( const Milliseconds millisecElapsed );
    void            Expire( void );  // zombify
	bool			IfExpired( void ) { return mZombie; }

private:
	Milliseconds	mTimeOfDeath;	///< when to destroy this particle system
	bool			mZombie;		///< if expired
};

} // namespace fx

#endif // FX_FX_PARTSYS_HH
