/*****************************************************************************
 * $LastChangedDate: 2008-11-09 22:02:22 -0500 (Sun, 09 Nov 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Trail particle-system (private implementation of FX class).
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef FX_FX_PARTSYS_TRAIL_HH
#define FX_FX_PARTSYS_TRAIL_HH 1

#include "fx/partsys.hh"
#include "fx/fx_partsys.hh"

namespace fx {

////////////////////////////////////////////////////////////////////////////////
/// @brief Trail particle.
///
class TrailParticle : public Particle
{
PREVENT_COPYING(TrailParticle)
typedef Particle Parent;
public:
					TrailParticle( const Sprite::Args& spriteArgs );
					~TrailParticle();
	virtual void	Tick( const Milliseconds millisecElapsed );
};

////////////////////////////////////////////////////////////////////////////////
/// @brief Trail particle-system (private implementation of FX class).
///
class TrailParticleSystemFX : public ParticleSystemFX
{
PREVENT_COPYING(TrailParticleSystemFX)
typedef ParticleSystemFX Parent;
friend class FX;
private:
					TrailParticleSystemFX( shptr<Object> trailMaker,
                                           const fp radius,
                                           const Milliseconds lifetime );
					~TrailParticleSystemFX();
	virtual void	Tick( const Milliseconds millisecElapsed );
	void			MakeParticles( const uint cnt );

private:
	shptr<Object>               mTrailMaker;        ///< emitter of particles
    WorldVertex                 mTrailMakerPrevPos; ///< prev position of trail maker
	const fp					mRadius;			///< radius of particle
	const Milliseconds			mTimeOfDeath;		///< time when particle system expires
	Milliseconds				mTimeOfPrevTick;	///< previous tick
	const RefPtr<osg::StateSet>	mStateSet;			///< particles share one stateset
};

} // namespace fx

#endif // FX_FX_PARTSYS_TRAIL_HH
