/*****************************************************************************
 * $LastChangedDate: 2008-11-12 01:09:43 -0500 (Wed, 12 Nov 2008) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
#include "base/timer.hh"
using namespace base;
#include "object/module.hh"
#include "object/dyna.hh"
using namespace object;
#include "game/module.hh"
#include "game/missile.hh"
using namespace game;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Missile"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================
// Need a function to check typesig because of macro/template limitations.
#if ! DOXYGEN
INTERN void
CheckTypesigMissile( shptr<Missile> obj )
{
    CHECK_TYPESIG(obj,TYPESIG_MISSILE);
}
#endif // ! DOXYGEN

/*****************************************************************************
 * Lua's garbage collector indirectly invokes this C++ destructor.
 *****************************************************************************/
INTERN int 
Missile_Destroy( lua_State* L ) 
{ 
   try { 

        return DESTROY_USERDATA_SHPTR<Missile>( L, CheckTypesigMissile );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Missile:new( target, launcher, launcherSpeed, radiusGeo, malfunction )
 *****************************************************************************/
INTERN int 
Missile_New( lua_State* L ) 
{ 
   try { 

		const bool malfunction        = POP_BOOL( L );
		const Meter particleRadiusGeo = Meter( POP_FP( L ) );
		const Speed launcherSpeed     = Speed( POP_FP( L ) );
        shptr<Dyna> launcher          = POP_USERDATA_AS_SHPTR<Dyna>( L );
        shptr<Dyna> target            = POP_USERDATA_AS_SHPTR<Dyna>( L );
		CHECK_TYPESIG(launcher,TYPESIG_OBJECT);
		CHECK_TYPESIG(target,TYPESIG_OBJECT);
		shptr<Missile> missile = new Missile( target, *launcher, launcherSpeed, particleRadiusGeo, malfunction );
        return NEW_USERDATA_SHPTR<Missile>( L, missile, BIND_METATABLE_NAME );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Missile:Rotate( axis, radian )
 *****************************************************************************/
INTERN int 
Missile_Rotate( lua_State* L ) 
{ 
   try { 

		const fp   radian = POP_FP( L );
		const uint axis   = POP_INT( L );
		shptr<Missile> missile = POP_USERDATA_AS_SHPTR<Missile>( L );
		CHECK_TYPESIG(missile,TYPESIG_MISSILE);
		missile->Rotate( axis, radian );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  { "new", Missile_New },
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  { "__gc", Missile_Destroy },
  { "Rotate", Missile_Rotate },
  { NULL, NULL }
};

void
RegisterMissile( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
